//*****************************************************************************
//
// File Name	: 'programmer.c'
// Title		: TX Programboard CT6B - using Microcontroller ATMEGA8
//            	  LCD Display 16x2 LCD44780 
// Author		: REY Copyright (C) 2012 based on Evzen Supler Programmer Transmitter - Copyright (C) 2011
// Created		: 2012.03.30 REY 
// Revised		: 2012.04.14 REY added  Heli Menu Items, Calibration, Backlight Switch
// Version		: 405
// Target MCU	: Atmel AVR Series  ATMEGA8
//
// NOTE: This code is currently below version 0.1, and therefore is considered
// to be lacking in some functionality or documentation, or may not be fully
// tested.  Nonetheless, you can expect most functions to work.
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************

/*

Menu:
Button UP    - move submenu to menu
Button Down  - move menu to submenu
Button Left  - move previous menu/submenu  and dec value
Button Right - move next menu/submenu  and inc value
Button OK State 0-3
          - 1x  - to settings value - step 1
          - 2x  - to settings value - step 5
          - 3x  - to settings value - step 25
          - 4x  - step = 0 return to submenu
State 0 move menu/submenu
State >0 setting value

Index  = MN.subMenuNo
Index  = MN.menuNo


*/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "lcd.h"
#include "globals.h"
#include "keyboard.h"
#include "menu.h"



#define MODELSTICK 2
#define STEP  5
#define DROFSET 4
#define SWASAFOFSET1 10
#define SWASAFOFSET2 11
#define SWASAFOFSET6 12
#define ENDRATEOFSET 13
#define THROEP0OFSET 25 // +IDT (0=NORM or 1=IDT)
#define THROEP1OFSET 27
#define THROEP2OFSET 29
#define THROEP3OFSET 31
#define THROEP4OFSET 33
#define PITHEP0OFSET 35 // +IDP (0=NORM or 1=IDT)
#define PITHEP1OFSET 37
#define PITHEP2OFSET 39
#define PITHEP3OFSET 41
#define PITHEP4OFSET 43
#define SUBTRIMOFSET 45
#define MIX1SDOFSET 51
#define MIX1UROFSET 52
#define MIX1DROFSET 53
#define MIX1SWOFSET 54
#define MIX2OFSET 4   // MIX1xxOFSET + MIX2OFSET
#define MIX3OFSET 8	  // MIX1xxOFSET + MIX3OFSET	
#define SWAVALOFSET 63
#define SWBVALOFSET 64
#define SWAPITADJ 65
#define SWBPITADJ 66


#define HEADER   85
#define HEADER_N 252  // normal data every 20ms
#define HEADER_R 253  // receiving data from device
#define HEADER_S 255  // send data to device  

#define STOPADJ  136  // calibration stop adjust
#define STRTADJ  153  // calibration start adjust
#define QUERY    250  // query for data from device

 

#define BAUD_RATE   115200

uint8_t data[69];
uint8_t EEMEM data0[69];
uint8_t EEMEM data1[69];
uint8_t EEMEM data2[69];
uint8_t EEMEM data3[69];
uint8_t EEMEM data4[69];
uint8_t EEMEM data5[69];
uint8_t EEMEM lcdbacklight;
int8_t num;

uint8_t MODELTYPE = 0;
// MODELTYPE for #menuitems pointer and TYPE/MODEL selection
// uint8_t CALIBRATE = 0;
// CALIBRATE Toggle
uint8_t IDT=0;
// ID  for Throttle Curve offset
uint8_t IDP=0;
// ID  for PITCH Curve offset

uint8_t State = 0;
// State  corresponds to MN.menuNo
// State = 0; browse mode
// State = 1; edit mode + step
// definition menu

uint8_t c,c1,Backlight;
int8_t i,j,Flag;
//actual model No
int8_t currModel=0;
// i menu j submenu
uint16_t sum;
// value D/R ch1on ch1off ch2on ch2off ch4on ch4off



// data definition 
// stick     start -     2 cnt 1  data[2]
// reverse   start -     3 cnt 1  data[3]
// dual rate start -     4 cnt 6  data[4-9]
// swash afr start -    10 cnt 3  data[10-12]
// end rate  start -    13 cnt 12 data[13-24]
// throttle curve start 25 cnt 10 data[25-34] 
// pitch curve    start 35 cnt 10 data[35-44]
// subtrim   start -    45 cnt 6  data[45-50]
// mix1 sour/dest start 51 cnt 1  data[51]
// mix1 up rate start   52 cnt 1  data[52]
// mix1 down rate start 53 cnt 1  data[53]
// mix1 switch    start 54 cnt 1  data[54]
// mix2 sour/dest start 55 cnt 1  data[55]
// mix2 up rate start   56 cnt 1  data[56]
// mix2 down rate start 57 cnt 1  data[57]
// mix2 switch    start 58 cnt 1  data[58]
// mix3 sour/dest start 59 cnt 1  data[59]
// mix3 up rate start   60 cnt 1  data[60]
// mix3 down rate start 61 cnt 1  data[61]
// mix3 switch    start 62 cnt 1  data[62]
// switch fcn SWA start 63 cnt 1  data[63]
// switch fcn SWB start 64 cnt 1  data[64]
// var A 	      start 65 cnt 1  data[65]
// var B 	      start 66 cnt 1  data[66]
// checksum High byte 	          data[67]
// checksum Low byte              data[68]



// direction not left not right
enum directions direction = null;
void print_menu(void);
int8_t counter(int8_t min, int8_t max, int8_t value, int8_t step, int8_t smer);
int8_t switchval(int8_t d);    // call SWX MIXSW
int8_t incHbyte(uint8_t d);    // call MIXCH
int8_t incLbyte(uint8_t d);    // call MIXCH
int8_t checkcrc17(void);
int checkcrc69(void);
void sumCrc(void);
void statusCrc69(void);

void init_Uart(void);
uint8_t USART_readByte(void);
void USART_writeByte( uint8_t c);
void USART_query(void);
void USART_stopadjust(void);
void USART_startadjust(void);

int initRadio(void);
void SaveToEeprom(void);
void LoadFromEeprom(void);


void func101(void);
void func102(void);
void func103(void);
void func104(void);
void func105(void);

void func201(void);
// func 202 - 206 not needed  
void func301(void);

void func401(void);

void func501(void);

void func601(void);
void func602(void);
void func603(void);
void func604(void);
void func605(void);

void func701(void);
void func702(void);
void func703(void);
void func704(void);
void func705(void);

void func801(void);
void func802(void);
void func803(void);
void func804(void);
void func805(void);

void func901(void);
void func1001(void);
void func1101(void);
void func1201(void);
void func1301(void);
void func1401(void);
void func1501(void);

void func1601(void);
void func1602(void);

void func1701(void);
void func1702(void);
void func1703(void);
void func1704(void);
void func1705(void);
void func1706(void);

void func1801(void);
void func1802(void);
void func1803(void);
void func1804(void);
void func1805(void);
void func1806(void);

void func1901(void);
void func1902(void);
void func1903(void);


const FuncPtr FuncPtrTable[] PROGMEM= {
func101, func102, func103,func104,func105, 		        //4 submenus of menu 1
func201, func201, func201,func201,func201,func201,      // can only be defined with a call to a function parameter in the index submenu
// func201, func202, func203,func204,func205,func206, 	// 6 submenus of menu 2
func301, func301, func301,func301,func301,func301, 		// 6 submenus of menu 3
func401, func401, func401,func401,func401,func401, 	
func401, func401, func401,func401,func401,func401, 		//12 submenus of menu 4
func501, func501, func501,func501,func501,func501, 		// 6 submenus of menu 5
func601, func602, func603,func604,func605, 		   	 	// 5 submenus of menu 6
func701, func702, func703,func704,func705, 	    		// 5 submenus of menu 7
func801, func802, func803,func804,func805, 	    		// 5 submenus of menu 8
func901,                                            	// 1 submenu  of menu 9  
func1001,                                           	// 1 submenu  of menu 10
func1101,                                           	// 1 submenu  of menu 11 
func1201,                                           	// 1 submenu  of menu 12 
func1301,                                           	// 1 submenu  of menu 13 
func1401,                                            	// 1 submenu  of menu 14 
func1501,												// 1 submenu  of menu 15 
func1601,func1602,									    // 2 submenus of menu 16 
func1701,func1702,func1703,func1704,func1705,func1706,	// 6 submenus of menu 17 
func1801,func1802,func1803,func1804,func1805,func1806,	// 6 submenus of menu 18 
func1901,func1902,func1903								// 3 submenus of menu 19 
		                                    
};


//Menu structure
//[0] -Number of level 0 menu items
//[1]...[n] number of second level menu items
//Eg. MSTR2[1] shows that menu item 1 has 3 submenus

const uint8_t MSTR2[] PROGMEM ={
	16, // number of menu items also depends on MODELTYPE 
	5,  // setup cnt MN101   Setup state
	6,  // of menu item  2   Reverse state 
	6,  // of menu item  3   Dr/ Ch 1,2,4 state 
	12, // of menu item  4   End Rate CH1-6 ON OFF state 
   	6,  // of menu item  5   Subtrim state
   	5,  // of menu item  6   Mix 1
   	5,  // of menu item  7   Mix 2
   	5,  // of menu item  8   Mix 3
	1,  // of menu item  9   FCN SWA
	1,  // of menu item  10  FCN SWB
	1,  // of menu item  11  VR-A
	1,  // of menu item  12	 VR-B 
	1,  // of menu item  13	 Type
	1,  // of menu item  14	 Stick
    1,  // of menu item  15  Backlight
	2,  // of menu item  16  Calibration	
    6,  // of menu item  17	 Thro Curve (HELI MODELTYPE=3) 
    6,  // of menu item  18	 Pith Curve (HELI MODELTYPE=3)  
    3,  // of menu item  19	 Swash Afr  (HELI MODELTYPE=3)
};  


uint8_t MFIndex(uint8_t mn, uint8_t sb)
// index = MFIIndex(menu, submenu)
{
	uint8_t p=0;//points to menu in table of function pointer 
	for(uint8_t i=0; i<(mn-1); i++)
	{
		p=p+pgm_read_byte(&MSTR2[i+1]);
	}
	p=p+sb-1;
	return p;
}



ISR(TIMER0_OVF_vect)
{
    cli();
	// Pause control push buttons 

	if (bit_is_clear(KBD_PIN, KBD_PIN_RIGHT))
	{
	   if (State==0) 
	   { 
		if (MN.subMenuNo<pgm_read_byte(&MSTR2[MN.menuNo]))
		{ 
			MN.subMenuNo++;
		}
		else
		{
			MN.subMenuNo=1;
		}

		} // State=0
	
		direction = plus;
		print_menu();
		loop_until_bit_is_set(KBD_PIN, KBD_PIN_RIGHT);


  	}

	if (bit_is_clear(KBD_PIN, KBD_PIN_LEFT))
	{
		if (State==0)
		{
		if (MN.subMenuNo==1)
		{ 
			MN.subMenuNo=pgm_read_byte(&MSTR2[MN.menuNo]);
		}
		else
		{
			MN.subMenuNo--;
		}
		} // State=0
				
		direction = minus;
		print_menu();
		loop_until_bit_is_set(KBD_PIN, KBD_PIN_LEFT);
	}

    if (bit_is_clear(KBD_PIN, KBD_PIN_DOWN))
	{
		direction = null;
		// cancel State > 0;
		if (State>0) State =0;

		if (MN.menuNo==1)
		{ 
			MN.menuNo=pgm_read_byte(&MSTR2[0]) + MODELTYPE;
			MN.subMenuNo=1;
		}
		else
		{
			MN.menuNo--;
		}
        print_menu();
		loop_until_bit_is_set(KBD_PIN, KBD_PIN_DOWN);
	}

    if (bit_is_clear(KBD_PIN, KBD_PIN_UP))
	{
	   	direction = null;
		if (MN.menuNo<(pgm_read_byte(&MSTR2[0]) + MODELTYPE)) 
		{ 
			MN.menuNo++;
			MN.subMenuNo=1;
		}
		else
		{
			MN.menuNo=1;
		}
        print_menu();

		loop_until_bit_is_set(KBD_PIN, KBD_PIN_DOWN);
	}

    if (bit_is_clear(KBD_PIN, KBD_PIN_OK))
	{
	    // set step 
		// 1x OK = step 1
		// 2x OK = step 5
		// 3x OK = step 25
		// 4x OK = step 0 
		if (State == 0)
		{ 
		   State++;
		}
		else 
		{
		  State = State * STEP;
		}
		if (State > 25) State = 0;
		direction = null;
		print_menu();
		loop_until_bit_is_set(KBD_PIN, KBD_PIN_OK);
	}

    sei();
}



void print_menu()
{
		lcd_clrscr();
        _delay_ms(200);
		lcd_puts_pxy(0,0,MENU[MN.menuNo-1]);
		lcd_puts_pxy(0,1,SUBMENU[MFIndex(MN.menuNo, MN.subMenuNo)]);
		FPtr=(FuncPtr)pgm_read_word(&FuncPtrTable[MFIndex(MN.menuNo, MN.subMenuNo)]);	
   //     Flag = 0;
		FPtr();		

}

void func101(void)
{
	if (State > 0)    
	{
		State = 0;
		currModel++ ;
		if (currModel == 6) currModel=0; 
	}
	lcd_puts_p(models[currModel]);
}

void func102(void)
// load from eeprom
{
//  lcd_putintxy(0,13,State,2);
  if (State >0)
  {	
  LoadFromEeprom();
  State = 0;
  statusCrc69();
  }
}
void func103(void)
// save to eeprom
{
  if (State >0)
  {	
  sumCrc();
  SaveToEeprom();
  State = 0;
  statusCrc69();
  }

}

void func104(void)
// load data from TX

//    lcd_putintxy(13,0,State,2);
{
    if (State > 0)
	{
	 cli();
     LEDPORT &= ~(1<<LEDERR);
     USART_query();


//	lcd_putintxy(13,0,State,2);
	i=0;
    while(1)
	{
	 c= USART_readByte(); 
	 if (c == HEADER) 
	 {
	 data[0] = c;
     // GET NEXT BYTE AND IF HEADER BREAK  
     	 c = USART_readByte();
    	 data[1] = c;
   		 if (c == HEADER_R) break;
//     break;	 
	 }
      i++;
	  if (i> 10) 
	  {
	    i=0;
		USART_query();
	  } 
	} // while(1) 
    for (i=2; i<69; i++)
	{
      c = USART_readByte();
		  data[i] = c ;
	  //  eeprom_write_byte((uint8_t*)i,c);  
	}  
    
    statusCrc69(); 
	State =0;	
	sei();
    if ((data[MODELSTICK] & 0x03)>0) MODELTYPE=3;
	else MODELTYPE =0;
	    
    } // end if state >0 
	// lcd_putc('R');
}

void func105(void)
// save to Transmitter
{
  if (State>0)
  {

  // modify checksum
  cli();
  
  sumCrc();
  // write to uart
  data[1] = HEADER_S;
  for (i=0; i<69; i++)
  {
  		c= data[i];
	    USART_writeByte(c); 
  } 

  statusCrc69();
 // lcd_putintxy(13,0,data[67],2);
 // lcd_putintxy(12,1,data[68],4);
 // lcd_puts_p(MNOK);
  State=0; 
  sei();
  }
}
void func201(void)  // reverse CH1
{
// int8_t p =	MN.subMenuNo-1 ;
if (State > 0)
  {
	 State=0;  // shodit enter
	 data[3] ^= (1<<(MN.subMenuNo-1));
  }
	if (bit_is_clear(data[3],MN.subMenuNo-1)) lcd_putc('N');
  else lcd_putc('R');

}

// dual rate ch1, ch2, ch4
void func301(void)
{

    data[DROFSET+MN.subMenuNo-1] = counter(0, +100, data[DROFSET+MN.subMenuNo-1], State, direction);  
    lcd_gotoxy(8,1);
	lcd_putint((data[DROFSET+MN.subMenuNo-1]),3);
//  lcd_putintxy(13,0,State,2);
    
}
// end rate

void func401(void)
{

    data[ENDRATEOFSET+MN.subMenuNo-1] = counter(0, +120, data[ENDRATEOFSET+MN.subMenuNo-1], State, direction);  
    lcd_gotoxy(5,2);  
	lcd_putint((data[ENDRATEOFSET+MN.subMenuNo-1]),3);
//  lcd_putintxy(13,0,State,2);
}

//subtrim
void func501(void)
{
    num = data[SUBTRIMOFSET+MN.subMenuNo-1];
	num = counter(-120, +120, num, State, direction);  
    data[SUBTRIMOFSET+MN.subMenuNo-1] = num; 
    lcd_gotoxy(4,1);
    lcd_putint(num,4); 
	if (num > 0)
	{
      lcd_gotoxy(4,1);
      lcd_putc('+'); 

	}

	lcd_gotoxy(7,1);
}



void func601(void)
{
// MIX1 tab[51]  high nible=source
   	data[MIX1SDOFSET] = incHbyte(data[MIX1SDOFSET]);  
  	lcd_puts_p(MIXCH[data[MIX1SDOFSET]/16]);
 	// lcd_putintxy(12,0,data[51],2);

}

void func602(void)
// MIX1 tab[51] low nible=destinat 
{
   	data[MIX1SDOFSET] = incLbyte(data[MIX1SDOFSET]);	
 	lcd_puts_p(MIXCH[data[MIX1SDOFSET] & 0x0F]);
}

void func603(void)
// tab[52] MIX1 Up rate
{
  data[MIX1UROFSET] = counter(-100, +100, data[MIX1UROFSET], State, direction); 
  lcd_gotoxy(9,1); 
  lcd_putint(data[MIX1UROFSET],4);
}

void func604(void)
//  tab[53] MIX1 Down rate
{
  data[MIX1DROFSET] = counter(-100, +100, data[MIX1DROFSET], State, direction); 
  lcd_gotoxy(9,1); 
  lcd_putint(data[MIX1DROFSET],4);
}

void func605(void)
{
    data[MIX1SWOFSET] = switchval(data[MIX1SWOFSET]);
	lcd_puts_p(MIXSW[data[MIX1SWOFSET]]);
}

void func701(void)
// MIX2 tab[55]  high nible=source
//0..5=ch1..ch6 (6..7=VRA..VRB >source only)
{
   	data[MIX1SDOFSET+MIX2OFSET] = incHbyte(data[MIX1SDOFSET+MIX2OFSET]);
	//  c=data[MIX1SDOFSET+MIX2OFSET]/16;
 	lcd_puts_p(MIXCH[data[MIX1SDOFSET+MIX2OFSET]/16]);
}

void func702(void)
// MIX2 tab[55]  low nible=dest
{
   	data[MIX1SDOFSET+MIX2OFSET] = incLbyte(data[MIX1SDOFSET+MIX2OFSET]);
 	lcd_puts_p(MIXCH[data[MIX1SDOFSET+MIX2OFSET] & 0x0F]);
}

void func703(void)
// MIX2 UR
{
    data[MIX1UROFSET+MIX2OFSET] = counter(-100, +100, data[MIX1UROFSET+MIX2OFSET], State, direction); 
    lcd_gotoxy(9,1); 

    lcd_putint(data[MIX1UROFSET+MIX2OFSET],4);
}

void func704(void)
// MIX2 DR
{
    data[MIX1DROFSET+MIX2OFSET] = counter(-100, +100, data[MIX1DROFSET+MIX2OFSET], State, direction); 
    lcd_gotoxy(9,1); 
    lcd_putint(data[MIX1DROFSET+MIX2OFSET],4);
}

void func705(void)
// MIX2 Switch 
{
	data[MIX1SWOFSET+MIX2OFSET] = switchval(data[MIX1SWOFSET+MIX2OFSET]);
	lcd_puts_p(MIXSW[data[MIX1SWOFSET+MIX2OFSET]]);
}


void func801(void)
// MIX3 source
{
   	data[MIX1SDOFSET+MIX3OFSET] = incHbyte(data[MIX1SDOFSET+MIX3OFSET]);   
  	lcd_puts_p(MIXCH[data[MIX1SDOFSET+MIX3OFSET]/16]);
}

void func802(void)
// MIX3 dest
{
	data[MIX1SDOFSET+MIX3OFSET] = incLbyte(data[MIX1SDOFSET+MIX3OFSET]);
    //c=data[MIX1SDOFSET+MIX3OFSET] & 0x0F;
 	lcd_puts_p(MIXCH[data[MIX1SDOFSET+MIX3OFSET] & 0x0F]);
}

void func803(void)
// MIX3 UR +-
{
    data[MIX1UROFSET+MIX3OFSET] = counter(-100, +100, data[MIX1UROFSET+MIX3OFSET], State, direction); 
    lcd_gotoxy(9,1); 
    lcd_putint(data[MIX1UROFSET+MIX3OFSET],4);
}

void func804(void)
// MIX3 DR +-
{
    data[MIX1DROFSET+MIX3OFSET] = counter(-100, +100, data[MIX1DROFSET+MIX3OFSET], State, direction); 
    lcd_gotoxy(9,1); 
    lcd_putint(data[MIX1DROFSET+MIX3OFSET],4);
}

void func805(void)
// MIX3 Switch 
{
	data[MIX1SWOFSET+MIX3OFSET] = switchval(data[MIX1SWOFSET+MIX3OFSET]);
	lcd_puts_p(MIXSW[data[MIX1SWOFSET+MIX3OFSET]]);
}


void func901(void)
// SWA var A
{
        data[SWAVALOFSET]= switchval(data[SWAVALOFSET]);
		lcd_puts_p(SWX[data[SWAVALOFSET]]);
}


void func1001(void)
// SWB var B
{
    data[SWBVALOFSET] = switchval(data[SWBVALOFSET]); 
	lcd_puts_p(SWX[data[SWBVALOFSET]]);
}

void func1101(void)
// SWA pit adj
{
	if (State > 0)    
	{
		State = 0;
		data[SWAPITADJ]++ ;
		if (data[SWAPITADJ] >= 2) data[SWAPITADJ] = 0; 

	}
	lcd_puts_p(MSV[data[SWAPITADJ]]);
}

void func1201(void)
// SWB pit adj
{
	if (State > 0)    
	{
		State = 0;
		data[SWBPITADJ]++; 
		if (data[SWBPITADJ] >=2) data[SWBPITADJ] = 0; 
	}
	lcd_puts_p(MSV[data[SWBPITADJ]]);
}

void func1301(void)
// Model Type 
{
	 // Model Type low nibble
    MODELTYPE=(data[MODELSTICK] & 0x03); 
	if (State > 0)    
	{
		State = 0;
		MODELTYPE++;
		if (MODELTYPE >=4) MODELTYPE = 0;
		data[MODELSTICK]=((data[MODELSTICK] & 0xF0) | MODELTYPE);
	}
	lcd_puts_p(MT[data[MODELSTICK] & 0x03]);
    if (MODELTYPE >0) MODELTYPE=3;
}

void func1401(void)
// Stick 
{
	uint8_t STICK=((data[MODELSTICK] & 0xF0)/16); //Stick Mode high nibble
	
	if (State > 0)    
	{
		State = 0;
		STICK=((data[MODELSTICK] & 0xF0)/16);
		STICK++;
		if (STICK >=4) STICK = 0;
		data[MODELSTICK]=((data[MODELSTICK] & 0x0F) + (STICK * 16));  
	}
    lcd_gotoxy(5,1); 
    lcd_putint((STICK+1),1);
	lcd_gotoxy(12,0);
    lcd_puts_p(STK0[(STICK & 1)]);
	lcd_gotoxy(11,1);
   	lcd_puts_p(STK1[(STICK >> 1)]);
}


void func1501(void)
// Backlight 
{
    if (State > 0)
	{
	   State = 0;	
	   Backlight++;
	   if (Backlight>=2) Backlight=0;
 	   eeprom_update_byte((uint8_t *)&lcdbacklight, Backlight);	
	}
    if (Backlight>0) LEDPORT |= (1<<LEDBL);
    else LEDPORT &= ~(1<<LEDBL);
	lcd_puts_p(MIXSW[(Backlight+2)]);
}

/*
void func1601(void)
// calibration 
{
    if (State > 0)
	{
       cli();
       LEDPORT &= ~(1<<LEDERR);
       if (CALIBRATE > 0) 
	   	 USART_stopadjust();
	   else  
	     USART_startadjust();
       State = 0;	
	   sei();
	   CALIBRATE++;
	   if (CALIBRATE >= 2) CALIBRATE = 0; 
	}
    lcd_gotoxy(0,1);
    lcd_puts_p(CALI[CALIBRATE]);
}
*/

void func1601(void)
// Stop Calibration 
{
    if (State > 0)
	{
       State = 0;	
	   cli();
       LEDPORT &= ~(1<<LEDERR);
       USART_stopadjust();
	   sei();
	}
}

void func1602(void)
// Start Calibration 
{
    if (State > 0)
	{
       State = 0;	
	   cli();
       LEDPORT &= ~(1<<LEDERR);
       USART_startadjust();
       sei();
	 }
}

void func1701(void)
// Thro Curve MODEL
{
	if (State > 0)
  	{
	  State=0;  
	  IDT++;
	  if (IDT>=2) IDT=0;
    }
    lcd_puts_p(CRVE[IDT]);
}

void func1702(void)
// Thro Curve EP0
{
  data[(THROEP0OFSET + IDT)] = counter(0, +100, data[(THROEP0OFSET + IDT)], State, direction); 
  lcd_gotoxy(4,1); 
  lcd_putint(data[(THROEP0OFSET + IDT)],3); 
}

void func1703(void)
// Thro Curve EP1
{
  data[(THROEP1OFSET + IDT)] = counter(0, +100, data[(THROEP1OFSET + IDT)], State, direction); 
  lcd_gotoxy(4,1); 
  lcd_putint(data[(THROEP1OFSET + IDT)],3); 
}

void func1704(void)
// Thro Curve EP2
{
  data[(THROEP2OFSET + IDT)] = counter(0, +100, data[(THROEP2OFSET + IDT)], State, direction); 
  lcd_gotoxy(4,1); 
  lcd_putint(data[(THROEP2OFSET + IDT)],3); 
}

void func1705(void)
// Thro Curve EP3
{
  data[(THROEP3OFSET + IDT)] = counter(0, +100, data[(THROEP3OFSET + IDT)], State, direction); 
  lcd_gotoxy(4,1); 
  lcd_putint(data[(THROEP3OFSET + IDT)],3); 
}

void func1706(void)
// Thro Curve EP4
{
  data[(THROEP4OFSET + IDT)] = counter(0, +100, data[(THROEP4OFSET + IDT)], State, direction); 
  lcd_gotoxy(4,1); 
  lcd_putint(data[(THROEP4OFSET + IDT)],3); 
}


void func1801(void)
// Pitch Curve Pitch
{
	if (State > 0)
  	{
	  State=0;  
	  IDP++;
	  if (IDP>=2) IDP=0;
    }
    lcd_puts_p(CRVE[IDP]);
}

void func1802(void)
// Pitch Curve EP0
{
  data[(PITHEP0OFSET + IDP)] = counter(0, +100, data[(PITHEP0OFSET + IDP)], State, direction); 
  lcd_gotoxy(4,1); 
  lcd_putint(data[(PITHEP0OFSET + IDP)],3); 
}

void func1803(void)
// Pitch Curve EP1
{
  data[(PITHEP1OFSET + IDP)] = counter(0, +100, data[(PITHEP1OFSET + IDP)], State, direction); 
  lcd_gotoxy(4,1); 
  lcd_putint(data[(PITHEP1OFSET + IDP)],3);  
}

void func1804(void)
// Pitch Curve EP2
{
  data[(PITHEP2OFSET + IDP)] = counter(0, +100, data[(PITHEP2OFSET + IDP)], State, direction); 
  lcd_gotoxy(4,1); 
  lcd_putint(data[(PITHEP2OFSET + IDP)],3); 
}

void func1805(void)
// Pitch Curve EP3
{
  data[(PITHEP3OFSET + IDP)] = counter(0, +100, data[(PITHEP3OFSET + IDP)], State, direction); 
  lcd_gotoxy(4,1); 
  lcd_putint(data[(PITHEP3OFSET + IDP)],3);  
}

void func1806(void)
// Pitch Curve EP4
{
  data[(PITHEP4OFSET + IDP)] = counter(0, +100, data[(PITHEP4OFSET + IDP)], State, direction); 
  lcd_gotoxy(4,1); 
  lcd_putint(data[(PITHEP4OFSET + IDP)],3); 
}

void func1901(void)
// Swash Afr CH1
{
    data[SWASAFOFSET1] = counter(-100, +100, data[SWASAFOFSET1], State, direction);  
    lcd_gotoxy(4,1);
	lcd_putint(data[SWASAFOFSET1],4);
}

void func1902(void)
// Swash Afr CH2
{
    data[SWASAFOFSET2] = counter(-100, +100, data[SWASAFOFSET2], State, direction);  
    lcd_gotoxy(4,1);
	lcd_putint(data[SWASAFOFSET2],4);
}

void func1903(void)
// Swash Afr CH6
{
    data[SWASAFOFSET6] = counter(-100, +100, data[SWASAFOFSET6], State, direction);  
    lcd_gotoxy(4,1);
	lcd_putint(data[SWASAFOFSET6],4);
 }


int8_t counter(int8_t min, int8_t max, int8_t value, int8_t step, int8_t smer)
{
if (smer == plus) // plus
{
   for (i= 0; i<step; i++)
   {
      if (value < max) value++;
   }  // end for
} // end direction plus
if (smer == minus)  // direction minus
{  
   for ( i=step; i>0 ; i-- )
   {
     if (value > min) value--;
   } //end for

} // end direction minus
	direction = null;
	lcd_putsxy(12,0,"R");
	lcd_putint(State,2);
	// lcd_gotoxy(7,2);
	return value;
}

int8_t switchval(int8_t d)
{
   if (State >0)
   {
	  d++;
	  if (d == 4) d=0;
   }
   State = 0;
   return d;
}

int8_t incHbyte(uint8_t d)
{
   if (State > 0)
   {
    State = 0;
   	// store H byte
   	c = d/16;
   	// store L byte
   	c1 = d & 0x0F;
   	c++;
 	if (c == 8) c=0;
    d = (c * 16)+ c1;
	}
    return d;
}

int8_t incLbyte(uint8_t d)
{
    if (State > 0)
	{
   	State = 0;
   	// store H byte
   	c = d/16;
   	// store L byte
   	c1 = d & 0x0F;
	c1++;
	if (c1 == 8) c1=0;
    d = (c * 16)+ c1;
    }
	return d;
}

int initRadio(void)
{
    // disable interrupt
    cli();
    // start read 17 byte
    //    LEDPORT |= (1<<LEDERR); 
	i=0;
    while(1)
	{
	 c= USART_readByte(); 
	 if (c == HEADER) 
	 {
	     data[0] = c;
         //GET NEXT BYTE AND IF HEADER BREAK 
     	 c = USART_readByte();
    	 data[1] = c;
   		 if (c == HEADER_N) 
		 { 
	        for (i=2; i<18; i++)
			{
      			c = USART_readByte();
		  		data[i] = c ;
			}  
		 } 
		 if (checkcrc17() == 0) break; 
	   }
      i++;
	  if (i> 10) 
	  {
	    i=0;
 		_delay_ms(10);
	  } 
	} // while(1) 
      if (checkcrc17() == 0) 
      {
	       LEDPORTOK |= (1<< LEDOK); 
		// LEDPORT &= ~(1<< LEDERR);
        // lcd_puts_pxy(1,6,MNOK);        
	  }
      else
	  {
	       LEDPORT |= (1<<LEDERR); 
       //  lcd_puts_pxy(1,6,MNERRCH);
           sei();
		   return 1;
	  } 	      
      sei();
      return 0;
}  // end initRadio


int checkcrc69(void)
{
     sum = data[67] * 256;
     sum = sum + data[68]; 
     for (i=2; i<67; i++)
     {
        sum = sum - (uint8_t)data[i];
     }
     return sum;
}

int8_t checkcrc17(void)
{
     int8_t j;
     
	 sum = (uint8_t)data[16] * 256;
     sum = sum + (uint8_t)data[17]; 
     for (j=2; j<(16); j++)
     {
        sum = sum - (uint8_t)data[j];
     }
     return sum;
}

void sumCrc(void)
{
	sum = 0;
    for (i=2; i<67; i++)
    {
	   sum = sum + (uint8_t)data[i];
    }
    // hight byte
    data[67] = sum /256;
    //  data[67] = data[67] +1 ; 
    // low byte
    data[68] = sum % 256;
}

void USART_writeByte( uint8_t c)
{
   while(!(UCSRA & (1<<UDRE)));
   UDR = c;
//   return 0;
}

void USART_query(void)
{
    USART_writeByte(HEADER);
	USART_writeByte(QUERY);
	USART_writeByte(0);
}

void USART_startadjust(void)
{

    USART_writeByte(HEADER);
	USART_writeByte(STRTADJ);
	USART_writeByte(0);
}
void USART_stopadjust(void)
{
    USART_writeByte(HEADER);
	USART_writeByte(STOPADJ);
	USART_writeByte(0);
}

uint8_t USART_readByte(void)
{
  // Wait until a byte has been received
  while(!(UCSRA & (1<<RXC))) ;
  return UDR;
  // Return received data
}

void init_Uart(void)
{
  UBRRH = (((F_CPU/BAUD_RATE)/16)-1)>>8;		// set baud rate
  UBRRL = (((F_CPU/BAUD_RATE)/16)-1);
  
  /* Enable Tx and Rx */
  UCSRB = (1<<RXEN)|(1<<TXEN);

  /* Set Frame: Data 8 Bit, No Parity and 1 Stop Bit */
  UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);
  // init LED ports
}

void SaveToEeprom(void)
{
  // save 69 bytes to eeprom and in case reset recall
  // values
  // Test invalid values! 
  // 1 par source 2 par dest 
    switch (currModel)
   {
 	  case 0:  eeprom_update_block((void*)&data, (const void*)&data0, 69); 
 	   	 break;
 	  case 1:  eeprom_update_block((void*)&data, (const void*)&data1, 69); 
 		 break;
	  case 2:  eeprom_update_block((void*)&data, (const void*)&data2, 69); 
 		 break;
	  case 3:  eeprom_update_block((void*)&data, (const void*)&data3, 69); 
 		 break;
	  case 4:  eeprom_update_block((void*)&data, (const void*)&data4, 69); 
 		 break;
	  case 5:  eeprom_update_block((void*)&data, (const void*)&data5, 69); 
 	 	 break;
    } //end case 
} // end void SaveToEeprom(void)

void LoadFromEeprom(void)
{
    // dest 2 par source
    switch (currModel)
 	{
      case 0:	eeprom_read_block((void*)&data,(const void*)&data0,69);
		 break;
  	  case 1:	eeprom_read_block((void*)&data,(const void*)&data1,69);
	     break;
  	  case 2:	eeprom_read_block((void*)&data,(const void*)&data2,69);
	     break;
  	  case 3:	eeprom_read_block((void*)&data,(const void*)&data3,69);
	     break;
  	  case 4:	eeprom_read_block((void*)&data,(const void*)&data4,69);
	     break;
  	  case 5:	eeprom_read_block((void*)&data,(const void*)&data5,69);
	     break;
	}  //end case
    statusCrc69();
}  // end LoadFromEeprom(void)


void statusCrc69(void)
{
	if ((checkcrc69() == 0) & (data[0] == HEADER)) 
	{
	  	LEDPORTOK |= (1<< LEDOK);
		LEDPORT &= ~(1<<LEDERR);
		lcd_puts_p(MNOK);
	}
    else 
	{
		LEDPORT |= (1<<LEDERR);
		LEDPORTOK &= ~(1<<LEDOK);
		lcd_puts_p(MNERR); 
	}
}

void backlightinit(void) //(uint8_t Backlight)
{
   Backlight = eeprom_read_byte((uint8_t *)&lcdbacklight);
   if (Backlight>2) 
   {
     Backlight=0;
   }  
   if (Backlight>0) LEDPORT |= (1<<LEDBL);
   else LEDPORT &= ~(1<<LEDBL);
}

int main(void)
{
	State=0;
    // init KBD & LED ports
	KBD_init();
	lcd_init(LCD_DISP_ON);
	lcd_clrscr();
	backlightinit();
	init_Uart();
    lcd_puts_pxy(1,0,MN000);
    _delay_ms(1500);
    lcd_clrscr();
    // wait until connect RADIO
    lcd_puts_pxy(0,0,INTRO1);
    lcd_puts_pxy(0,1,INTRO2);
     
    if (initRadio() == 1)
    // ERR CHANNEL
	{
       lcd_clrscr();
       lcd_putsxy(1,2,STOPPROGRAM);
       lcd_home();
       return 1;
        // halt;
    }
	// init port
	lcd_gotoxy(0,1);
    State = 1;   // active func104
	func104();  // load from TX	
 	menu_init();
	print_menu();
	Timer0_init();
	while(1)
	{
	  Flag = 1;
	}
}

